@@ -35,6 +35,8 @@
 // Date: 23 May 2022
 //***************************************************************************/
 
+#include "ojph_arch.h"
+#if defined(OJPH_ARCH_I386) || defined(OJPH_ARCH_X86_64)
 
 #include <cstdlib>
 #include <cstring>
@@ -352,3 +354,5 @@ namespace ojph {
     }    
   }
 }
+
+#endif
