@@ -40,6 +40,9 @@
  *  @brief implements a faster HTJ2K block decoder using ssse3
  */
 
+#include "ojph_arch.h"
+#if defined(OJPH_ARCH_I386) || defined(OJPH_ARCH_X86_64)
+
 #include <string>
 #include <iostream>
 
@@ -47,7 +50,6 @@
 #include <cstring>
 #include "ojph_block_common.h"
 #include "ojph_block_decoder.h"
-#include "ojph_arch.h"
 #include "ojph_message.h"
 
 #include <immintrin.h>
@@ -2064,3 +2066,5 @@ namespace ojph {
     }
   }
 }
+
+#endif
